@@ -18,9 +18,8 @@ export LD_LIBRARY_PATH="$inst/lib"
 export PATH="$inst/bin:$PATH"
 
 osmo-build-dep.sh libosmocore "" --disable-doxygen
-osmo-build-dep.sh libosmo-abis
-osmo-build-dep.sh libosmo-netif
-osmo-build-dep.sh libosmo-sccp
+osmo-build-dep.sh libosmo-netif "" --disable-doxygen
+osmo-build-dep.sh libosmo-sigtran "" --disable-doxygen
 osmo-build-dep.sh libasn1c
 
 # the asn1c binary is used by the 'regen' target below
