@@ -78,7 +78,7 @@ static const struct value_string hnbap_cause_misc_vals[] = {
 	{ 0, NULL }
 };
 
-char *hnbap_cause_str(HNBAP_Cause_t *cause)
+char *hnbap_cause_str(const HNBAP_Cause_t *cause)
 {
 	static char buf[32];
 
@@ -103,6 +103,9 @@ char *hnbap_cause_str(HNBAP_Cause_t *cause)
 			get_value_string(hnbap_cause_misc_vals,
 					cause->choice.misc));
 		break;
+	case HNBAP_Cause_PR_NOTHING:
+		snprintf(buf, sizeof(buf), "NOTHING");
+		break;
 	}
 	return buf;
 }
@@ -149,7 +152,7 @@ struct msgb *hnbap_generate_initiating_message(
 	pdu.choice.initiatingMessage.criticality = criticality;
 	rc = ANY_fromType_aper(&pdu.choice.initiatingMessage.value, td, sptr);
 	if (rc < 0) {
-		LOGP(DHNBAP, LOGL_ERROR, "Error in ANY_fromType_aper\n");
+		LOGP(DHNBAP, LOGL_ERROR, "%s: Error in ANY_fromType_aper (%s)\n", __func__, td->name);
 		return NULL;
 	}
 
@@ -176,7 +179,7 @@ struct msgb *hnbap_generate_successful_outcome(
 	pdu.choice.successfulOutcome.criticality = criticality;
 	rc = ANY_fromType_aper(&pdu.choice.successfulOutcome.value, td, sptr);
 	if (rc < 0) {
-		LOGP(DHNBAP, LOGL_ERROR, "Error in ANY_fromType_aper\n");
+		LOGP(DHNBAP, LOGL_ERROR, "%s: Error in ANY_fromType_aper (%s)\n", __func__, td->name);
 		return NULL;
 	}
 
@@ -203,7 +206,7 @@ struct msgb *hnbap_generate_unsuccessful_outcome(
 	pdu.choice.unsuccessfulOutcome.criticality = criticality;
 	rc = ANY_fromType_aper(&pdu.choice.unsuccessfulOutcome.value, td, sptr);
 	if (rc < 0) {
-		LOGP(DHNBAP, LOGL_ERROR, "Error in ANY_fromType_aper\n");
+		LOGP(DHNBAP, LOGL_ERROR, "%s: Error in ANY_fromType_aper (%s)\n", __func__, td->name);
 		return NULL;
 	}
 
@@ -231,7 +234,7 @@ HNBAP_IE_t *hnbap_new_ie(HNBAP_ProtocolIE_ID_t id,
 
 	rc = ANY_fromType_aper(&buff->value, type, sptr);
 	if (rc < 0) {
-		LOGP(DHNBAP, LOGL_ERROR, "Error in ANY_fromType_aper\n");
+		LOGP(DHNBAP, LOGL_ERROR, "%s: Error in ANY_fromType_aper(%s)\n", __func__, type->name);
 		FREEMEM(buff);
 		return NULL;
 	}
