@@ -31,8 +31,7 @@
 
 #define DRANAP _ranap_DRANAP
 
-static int cn_ranap_rx_initiating_msg_co(void *ctx, RANAP_InitiatingMessage_t *imsg,
-					 ranap_message *message)
+static int cn_ranap_rx_initiating_msg_co(RANAP_InitiatingMessage_t *imsg, ranap_message *message)
 {
 	int rc = 0;
 
@@ -73,6 +72,20 @@ static int cn_ranap_rx_initiating_msg_co(void *ctx, RANAP_InitiatingMessage_t *i
 		/* FIXME this is not a message received by CN (used by hnb-test) */
 		rc = ranap_decode_iu_releasecommandies(&message->msg.iu_ReleaseCommandIEs, &imsg->value);
 		break;
+	case RANAP_ProcedureCode_id_RelocationPreparation: /* RELOCATION REQUIRED */
+		rc = ranap_decode_relocationrequiredies(&message->msg.relocationRequiredIEs, &imsg->value);
+		break;
+	//case RANAP_ProcedureCode_id_RelocationDetect: /* TODO: no member in ranap_message */
+	//case RANAP_ProcedureCode_id_RelocationComplete: /* TODO: no member in ranap_message */
+	case RANAP_ProcedureCode_id_RelocationCancel:
+		rc = ranap_decode_relocationcancelies(&message->msg.relocationCancelIEs, &imsg->value);
+		break;
+	case RANAP_ProcedureCode_id_ForwardSRNS_Context:
+		rc = ranap_decode_forwardsrns_contexties(&message->msg.forwardSRNS_ContextIEs, &imsg->value);
+		break;
+	case RANAP_ProcedureCode_id_LocationReport:
+		rc = ranap_decode_locationreporties(&message->msg.locationReportIEs, &imsg->value);
+		break;
 	default:
 		LOGP(DRANAP, LOGL_INFO,
 		     "Received RANAP Procedure %s (CO, IM) from RNC. Decode not implemented\n",
@@ -117,6 +130,20 @@ static void cn_ranap_free_initiating_msg_co(ranap_message *message)
 		/* FIXME this is not a message received by CN (used by hnb-test) */
 		ranap_free_iu_releasecommandies(&message->msg.iu_ReleaseCommandIEs);
 		break;
+	case RANAP_ProcedureCode_id_RelocationPreparation: /* RELOCATION REQUIRED */
+		ranap_free_relocationrequiredies(&message->msg.relocationRequiredIEs);
+		break;
+	//case RANAP_ProcedureCode_id_RelocationDetect: /* TODO: no member in ranap_message */
+	//case RANAP_ProcedureCode_id_RelocationComplete: /* TODO: no member in ranap_message */
+	case RANAP_ProcedureCode_id_RelocationCancel:
+		ranap_free_relocationcancelies(&message->msg.relocationCancelIEs);
+		break;
+	case RANAP_ProcedureCode_id_ForwardSRNS_Context:
+		ranap_free_forwardsrns_contexties(&message->msg.forwardSRNS_ContextIEs);
+		break;
+	case RANAP_ProcedureCode_id_LocationReport:
+		ranap_free_locationreporties(&message->msg.locationReportIEs);
+		break;
 	default:
 		LOGP(DRANAP, LOGL_INFO,
 		     "Freeing RANAP Procedure %s (CO, IM) from RNC not implemented\n",
@@ -125,8 +152,7 @@ static void cn_ranap_free_initiating_msg_co(ranap_message *message)
 	}
 }
 
-static int cn_ranap_rx_successful_msg_co(void *ctx, RANAP_SuccessfulOutcome_t *imsg,
-					 ranap_message *message)
+static int cn_ranap_rx_successful_msg_co(RANAP_SuccessfulOutcome_t *imsg, ranap_message *message)
 {
 	int rc = 0;
 
@@ -141,13 +167,25 @@ static int cn_ranap_rx_successful_msg_co(void *ctx, RANAP_SuccessfulOutcome_t *i
 		/* RAB assignment response */
 		rc = ranap_decode_rab_assignmentresponseies(&message->msg.raB_AssignmentResponseIEs, &imsg->value);
 		break;
+	case RANAP_ProcedureCode_id_Iu_Release:
+		/* Iu release Complete; confirmation of CN-initiated release */
+		rc = ranap_decode_iu_releasecompleteies(&message->msg.iu_ReleaseCompleteIEs, &imsg->value);
+		break;
+	case RANAP_ProcedureCode_id_RelocationResourceAllocation:
+		rc = ranap_decode_relocationrequestacknowledgeies(&message->msg.relocationRequestAcknowledgeIEs, &imsg->value);
+		break;
+	case RANAP_ProcedureCode_id_SRNS_ContextTransfer:
+		rc = ranap_decode_srns_contextresponseies(&message->msg.srnS_ContextResponseIEs, &imsg->value);
+		break;
 	case RANAP_ProcedureCode_id_SecurityModeControl:
 		/* Security Mode Complete */
 		rc = ranap_decode_securitymodecompleteies(&message->msg.securityModeCompleteIEs, &imsg->value);
 		break;
-	case RANAP_ProcedureCode_id_Iu_Release:
-		/* Iu release Complete; confirmation of CN-initiated release */
-		rc = ranap_decode_iu_releasecompleteies(&message->msg.iu_ReleaseCompleteIEs, &imsg->value);
+	case RANAP_ProcedureCode_id_DataVolumeReport:
+		rc = ranap_decode_datavolumereporties(&message->msg.dataVolumeReportIEs, &imsg->value);
+		break;
+	case RANAP_ProcedureCode_id_LocationRelatedData:
+		rc = ranap_decode_locationrelateddataresponseies(&message->msg.locationRelatedDataResponseIEs, &imsg->value);
 		break;
 	default:
 		LOGP(DRANAP, LOGL_INFO,
@@ -167,13 +205,25 @@ static void cn_ranap_free_successful_msg_co(ranap_message *message)
 		/* RAB assignment response */
 		ranap_free_rab_assignmentresponseies(&message->msg.raB_AssignmentResponseIEs);
 		break;
+	case RANAP_ProcedureCode_id_Iu_Release:
+		/* Iu release Complete; confirmation of CN-initiated release */
+		ranap_free_iu_releasecompleteies(&message->msg.iu_ReleaseCompleteIEs);
+		break;
+	case RANAP_ProcedureCode_id_RelocationResourceAllocation:
+		ranap_free_relocationrequestacknowledgeies(&message->msg.relocationRequestAcknowledgeIEs);
+		break;
+	case RANAP_ProcedureCode_id_SRNS_ContextTransfer:
+		ranap_free_srns_contextresponseies(&message->msg.srnS_ContextResponseIEs);
+		break;
 	case RANAP_ProcedureCode_id_SecurityModeControl:
 		/* Security Mode Complete */
 		ranap_free_securitymodecompleteies(&message->msg.securityModeCompleteIEs);
 		break;
-	case RANAP_ProcedureCode_id_Iu_Release:
-		/* Iu release Complete; confirmation of CN-initiated release */
-		ranap_free_iu_releasecompleteies(&message->msg.iu_ReleaseCompleteIEs);
+	case RANAP_ProcedureCode_id_DataVolumeReport:
+		ranap_free_datavolumereporties(&message->msg.dataVolumeReportIEs);
+		break;
+	case RANAP_ProcedureCode_id_LocationRelatedData:
+		ranap_free_locationrelateddataresponseies(&message->msg.locationRelatedDataResponseIEs);
 		break;
 	default:
 		LOGP(DRANAP, LOGL_INFO,
@@ -183,8 +233,7 @@ static void cn_ranap_free_successful_msg_co(ranap_message *message)
 	}
 }
 
-static int cn_ranap_rx_outcome_msg_co(void *ctx, RANAP_Outcome_t *imsg,
-					ranap_message *message)
+static int cn_ranap_rx_outcome_msg_co(RANAP_Outcome_t *imsg, ranap_message *message)
 {
 	int rc = 0;
 
@@ -225,26 +274,71 @@ static void cn_ranap_free_outcome_msg_co(ranap_message *message)
 	}
 }
 
-static int _cn_ranap_rx_co(void *ctx, RANAP_RANAP_PDU_t *pdu, ranap_message *message)
+static int cn_ranap_rx_unsuccessful_msg_co(RANAP_UnsuccessfulOutcome_t *imsg, ranap_message *message)
+{
+	int rc = 0;
+
+	message->procedureCode = imsg->procedureCode;
+	message->criticality = imsg->criticality;
+
+	DEBUGP(DRANAP, "Rx CO USO (%s)\n", get_value_string(ranap_procedure_code_vals, imsg->procedureCode));
+
+	switch (imsg->procedureCode) {
+	case RANAP_ProcedureCode_id_RelocationResourceAllocation:
+		rc = ranap_decode_relocationfailureies(&message->msg.relocationFailureIEs, &imsg->value);
+		break;
+	case RANAP_ProcedureCode_id_SecurityModeControl:
+		rc = ranap_decode_securitymoderejecties(&message->msg.securityModeRejectIEs, &imsg->value);
+		break;
+	case RANAP_ProcedureCode_id_LocationRelatedData:
+		rc = ranap_decode_locationrelateddatafailureies(&message->msg.locationRelatedDataFailureIEs, &imsg->value);
+		break;
+	default:
+		LOGP(DRANAP, LOGL_NOTICE, "Received unsupported RANAP "
+		     "unsuccessful outcome procedure %s (CO) from RNC, ignoring\n",
+		     get_value_string(ranap_procedure_code_vals, imsg->procedureCode));
+		rc = -1;
+		break;
+	}
+
+	return rc;
+}
+
+static void cn_ranap_free_unsuccessful_msg_co(ranap_message *message)
+{
+	switch (message->procedureCode) {
+	case RANAP_ProcedureCode_id_RelocationResourceAllocation:
+		ranap_free_relocationfailureies(&message->msg.relocationFailureIEs);
+		break;
+	case RANAP_ProcedureCode_id_SecurityModeControl:
+		ranap_free_securitymoderejecties(&message->msg.securityModeRejectIEs);
+		break;
+	case RANAP_ProcedureCode_id_LocationRelatedData:
+		ranap_free_locationrelateddatafailureies(&message->msg.locationRelatedDataFailureIEs);
+		break;
+	default:
+		LOGP(DRANAP, LOGL_INFO, "Freeing RANAP unsuccessful outcome procedure %s (CO) from RNC "
+		     "not implemented\n", get_value_string(ranap_procedure_code_vals, message->procedureCode));
+		break;
+	}
+}
+
+static int _cn_ranap_rx_co(RANAP_RANAP_PDU_t *pdu, ranap_message *message)
 {
 	int rc = 0;
 
 	switch (pdu->present) {
 	case RANAP_RANAP_PDU_PR_initiatingMessage:
-		rc = cn_ranap_rx_initiating_msg_co(ctx, &pdu->choice.initiatingMessage, message);
+		rc = cn_ranap_rx_initiating_msg_co(&pdu->choice.initiatingMessage, message);
 		break;
 	case RANAP_RANAP_PDU_PR_successfulOutcome:
-		rc = cn_ranap_rx_successful_msg_co(ctx, &pdu->choice.successfulOutcome, message);
+		rc = cn_ranap_rx_successful_msg_co(&pdu->choice.successfulOutcome, message);
 		break;
 	case RANAP_RANAP_PDU_PR_unsuccessfulOutcome:
-		LOGP(DRANAP, LOGL_NOTICE, "Received unsupported RANAP "
-		     "unsuccessful outcome procedure %s (CO) from RNC, ignoring\n",
-		     get_value_string(ranap_procedure_code_vals,
-			     	      pdu->choice.unsuccessfulOutcome.procedureCode));
-		rc = -1;
+		rc = cn_ranap_rx_unsuccessful_msg_co(&pdu->choice.unsuccessfulOutcome, message);
 		break;
 	case RANAP_RANAP_PDU_PR_outcome:
-		rc = cn_ranap_rx_outcome_msg_co(ctx, &pdu->choice.outcome, message);
+		rc = cn_ranap_rx_outcome_msg_co(&pdu->choice.outcome, message);
 		break;
 	default:
 		LOGP(DRANAP, LOGL_INFO,
@@ -268,8 +362,7 @@ void ranap_cn_rx_co_free(ranap_message *message)
 		cn_ranap_free_successful_msg_co(message);
 		break;
 	case RANAP_RANAP_PDU_PR_unsuccessfulOutcome:
-		LOGP(DRANAP, LOGL_INFO,
-		     "Freeing RANAP unsuccessful outcome procedure (CO) from RNC not implemented\n");
+		cn_ranap_free_unsuccessful_msg_co(message);
 		break;
 	case RANAP_RANAP_PDU_PR_outcome:
 		cn_ranap_free_outcome_msg_co(message);
@@ -283,7 +376,7 @@ void ranap_cn_rx_co_free(ranap_message *message)
 }
 
 /* decode a connection-oriented RANAP message */
-int ranap_cn_rx_co_decode(void *ctx, ranap_message *message, uint8_t *data, size_t len)
+int ranap_cn_rx_co_decode2(ranap_message *message, uint8_t *data, size_t len)
 {
 	RANAP_RANAP_PDU_t *pdu = NULL;
 	asn_dec_rval_t dec_ret;
@@ -294,40 +387,46 @@ int ranap_cn_rx_co_decode(void *ctx, ranap_message *message, uint8_t *data, size
 	dec_ret = aper_decode(NULL, &asn_DEF_RANAP_RANAP_PDU, (void **)&pdu, data, len, 0, 0);
 	if (dec_ret.code != RC_OK) {
 		LOGP(DRANAP, LOGL_ERROR, "Error in RANAP ASN.1 decode\n");
-		return -1;
+		rc = -1;
+		goto error_free;
 	}
 
 	message->direction = pdu->present;
 
-	rc = _cn_ranap_rx_co(ctx, pdu, message);
+	rc = _cn_ranap_rx_co(pdu, message);
 
+error_free:
 	ASN_STRUCT_FREE(asn_DEF_RANAP_RANAP_PDU, pdu);
 
 	return rc;
 }
 
+int ranap_cn_rx_co_decode(void *unused, ranap_message *message, uint8_t *data, size_t len)
+{
+	return ranap_cn_rx_co_decode2(message, data, len);
+}
+
 /* receive a connection-oriented RANAP message and call
  * cn_ranap_handle_co() with the resulting ranap_message struct */
-int ranap_cn_rx_co(ranap_handle_cb cb, void *ctx, uint8_t *data, size_t len)
+int ranap_cn_rx_co(ranap_handle_cb cb, void *priv, uint8_t *data, size_t len)
 {
 	ranap_message message;
 	int rc;
 
-	rc = ranap_cn_rx_co_decode(ctx, &message, data, len);
-
-	if (rc == 0)
-		(*cb)(ctx, &message);
-	else
+	rc = ranap_cn_rx_co_decode2(&message, data, len);
+	if (rc) {
 		LOGP(DRANAP, LOGL_ERROR, "Not calling cn_ranap_handle_co() due to rc=%d\n", rc);
+		return rc;
+	}
+
+	(*cb)(priv, &message);
 
 	/* Free the asn1 structs in message */
 	ranap_cn_rx_co_free(&message);
-
-	return rc;
+	return 0;
 }
 
-static int cn_ranap_rx_initiating_msg_cl(void *ctx, RANAP_InitiatingMessage_t *imsg,
-					 ranap_message *message)
+static int cn_ranap_rx_initiating_msg_cl(RANAP_InitiatingMessage_t *imsg, ranap_message *message)
 {
 	int rc = 0;
 
@@ -412,8 +511,7 @@ static void cn_ranap_free_initiating_msg_cl(ranap_message *message)
 	}
 }
 
-static int cn_ranap_rx_successful_msg_cl(void *ctx, RANAP_SuccessfulOutcome_t *imsg,
-					 ranap_message *message)
+static int cn_ranap_rx_successful_msg_cl(RANAP_SuccessfulOutcome_t *imsg, ranap_message *message)
 {
 	int rc = 0;
 
@@ -475,7 +573,7 @@ static void cn_ranap_free_successful_msg_cl(ranap_message *message)
 	}
 }
 
-static int _cn_ranap_rx_cl(void *ctx, RANAP_RANAP_PDU_t *pdu, ranap_message *message)
+static int _cn_ranap_rx_cl(RANAP_RANAP_PDU_t *pdu, ranap_message *message)
 {
 	int rc = 0;
 
@@ -483,12 +581,10 @@ static int _cn_ranap_rx_cl(void *ctx, RANAP_RANAP_PDU_t *pdu, ranap_message *mes
 
 	switch (pdu->present) {
 	case RANAP_RANAP_PDU_PR_initiatingMessage:
-		rc = cn_ranap_rx_initiating_msg_cl(ctx, &pdu->choice.initiatingMessage,
-						   message);
+		rc = cn_ranap_rx_initiating_msg_cl(&pdu->choice.initiatingMessage, message);
 		break;
 	case RANAP_RANAP_PDU_PR_successfulOutcome:
-		rc = cn_ranap_rx_successful_msg_cl(ctx, &pdu->choice.successfulOutcome,
-						   message);
+		rc = cn_ranap_rx_successful_msg_cl(&pdu->choice.successfulOutcome, message);
 		break;
 	case RANAP_RANAP_PDU_PR_unsuccessfulOutcome:
 		LOGP(DRANAP, LOGL_NOTICE, "Received unsupported RANAP "
@@ -528,7 +624,7 @@ void ranap_cn_rx_cl_free(ranap_message *message)
 }
 
 /* decode a connection-less RANAP message */
-int ranap_cn_rx_cl_decode(void *ctx, ranap_message *message, uint8_t *data, size_t len)
+int ranap_cn_rx_cl_decode2(ranap_message *message, uint8_t *data, size_t len)
 {
 	RANAP_RANAP_PDU_t *pdu = NULL;
 	asn_dec_rval_t dec_ret;
@@ -539,29 +635,36 @@ int ranap_cn_rx_cl_decode(void *ctx, ranap_message *message, uint8_t *data, size
 	dec_ret = aper_decode(NULL, &asn_DEF_RANAP_RANAP_PDU, (void **)&pdu, data, len, 0, 0);
 	if (dec_ret.code != RC_OK) {
 		LOGP(DRANAP, LOGL_ERROR, "Error in RANAP ASN.1 decode\n");
-		return -1;
+		rc = -1;
+		goto error_free;
 	}
 
 	message->direction = pdu->present;
 
-	rc = _cn_ranap_rx_cl(ctx, pdu, message);
+	rc = _cn_ranap_rx_cl(pdu, message);
 
+error_free:
 	ASN_STRUCT_FREE(asn_DEF_RANAP_RANAP_PDU, pdu);
 
 	return rc;
 }
 
+int ranap_cn_rx_cl_decode(void *unused, ranap_message *message, uint8_t *data, size_t len)
+{
+	return ranap_cn_rx_cl_decode2(message, data, len);
+}
+
 /* receive a connection-less RANAP message and call
  * cn_ranap_handle_co() with the resulting ranap_message struct */
-int ranap_cn_rx_cl(ranap_handle_cb cb, void *ctx, uint8_t *data, size_t len)
+int ranap_cn_rx_cl(ranap_handle_cb cb, void *priv, uint8_t *data, size_t len)
 {
 	ranap_message message;
 	int rc;
 
-	rc = ranap_cn_rx_cl_decode(ctx, &message, data, len);
+	rc = ranap_cn_rx_cl_decode2(&message, data, len);
 
 	if (rc == 0)
-		(*cb)(ctx, &message);
+		(*cb)(priv, &message);
 	else
 		LOGP(DRANAP, LOGL_ERROR, "Not calling cn_ranap_handle_cl() due to rc=%d\n", rc);
 
